using System;
using System.Runtime.InteropServices;
using System.Security;

namespace MyMemoryReadSearchLib {
    internal sealed class SafeProcessHandle : SafeHandle {
        public SafeProcessHandle() : base(IntPtr.Zero, true) { }
        public override bool IsInvalid { get { return base.handle == IntPtr.Zero; } }
        protected override bool ReleaseHandle() { return SafeNativeMethods.CloseHandle(base.handle); }
    }

    [SuppressUnmanagedCodeSecurityAttribute]
    internal static class SafeNativeMethods {
        [Flags] //WinNT.h
        internal enum PageAccessFlags : uint {
            PAGE_READWRITE = 0x00000004,
            MEM_COMMIT = 0x00001000,
            MEM_PRIVATE = 0x00020000
        }
        [Flags] //WinNT.h
        internal enum ProcessAccessFlags : uint {
            PROCESS_VM_READ = 0x00000010,
            PROCESS_QUERY_INFORMATION = 0x00000400
        }

        /*HANDLE WINAPI OpenProcess(
            DWORD dwDesiredAccess,
            BOOL bInheritHandle,
            DWORD dwProcessId
        );*/
        [DllImport("kernel32.dll")]
        internal static extern SafeProcessHandle OpenProcess(ProcessAccessFlags dwDesiredAccess, [MarshalAs(UnmanagedType.Bool)] bool bInheritHandle, uint dwProcessId);

        /*BOOL CloseHandle(
            HANDLE hObject
        );*/
        [DllImport("kernel32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        internal static extern bool CloseHandle(IntPtr hObject);

        /*BOOL ReadProcessMemory(
            HANDLE hProcess,
            LPCVOID lpBaseAddress,
            LPVOID lpBuffer,
            SIZE_T nSize,
            SIZE_T* lpNumberOfBytesRead
        );*/
        [DllImport("kernel32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        internal static extern bool ReadProcessMemory(SafeProcessHandle hProcess, IntPtr lpBaseAddress, [Out] byte[] lpBuffer, UIntPtr nSize, UIntPtr lpNumberOfBytesRead);

        /*SIZE_T VirtualQueryEx(
            HANDLE hProcess,
            LPCVOID lpAddress,
            PMEMORY_BASIC_INFORMATION lpBuffer,
            SIZE_T dwLength
        );*/
        [DllImport("kernel32.dll")]
        internal static extern UIntPtr VirtualQueryEx(SafeProcessHandle hProcess, IntPtr lpAddress, out MEMORY_BASIC_INFORMATION lpBuffer, UIntPtr dwLength);

        /**32BIT ONLY** void GetSystemInfo(
            LPSYSTEM_INFO lpSystemInfo
        );
        [DllImport("kernel32.dll")]
        internal static extern void GetSystemInfo(out SYSTEM_INFO lpSystemInfo);

        [StructLayout(LayoutKind.Sequential)]
            internal struct SYSTEM_INFO {
            internal _PROCESSOR_INFO_UNION p;
            public uint dwPageSize;
            public IntPtr lpMinimumApplicationAddress;
            public IntPtr lpMaximumApplicationAddress;
            public IntPtr dwActiveProcessorMask;
            public uint dwNumberOfProcessors;
            public uint dwProcessorType;
            public uint dwAllocationGranularity;
            public ushort wProcessorLevel;
            public ushort wProcessorRevision;
        }
        [StructLayout(LayoutKind.Explicit)]
            internal struct _PROCESSOR_INFO_UNION {
            [FieldOffset(0)] internal uint dwOemId;
            [FieldOffset(0)] internal ushort wProcessorArchitecture;
            [FieldOffset(2)] internal ushort wReserved;
        }*/

        [StructLayout(LayoutKind.Sequential)]
        internal struct MEMORY_BASIC_INFORMATION {
            public IntPtr baseAddress;
            public IntPtr allocationBase;
            public uint allocationProtect;
            public UIntPtr regionSize;
            public PageAccessFlags state;
            public PageAccessFlags protect;
            public PageAccessFlags type;
        }
    }

    public class MyMemoryReadSearch : IDisposable {
        //**32BIT ONLY** private SYSTEM_INFO sys_info;
        private SafeProcessHandle m_hProcess;

        //**32BIT ONLY** public ProcMemReadSearch() { GetSystemInfo(out sys_info); }
        public void OpenProcess(uint processId) {
            if(m_hProcess != null && !m_hProcess.IsClosed) m_hProcess.Dispose();
            m_hProcess = SafeNativeMethods.OpenProcess(SafeNativeMethods.ProcessAccessFlags.PROCESS_QUERY_INFORMATION | SafeNativeMethods.ProcessAccessFlags.PROCESS_VM_READ, true, processId);
        }
        public byte[] ReadProcessMemory(IntPtr baseAddress, UIntPtr size) {
            if(disposed) return null;
            byte[] ReturnBuffer = new byte[(ulong)size];
            return SafeNativeMethods.ReadProcessMemory(m_hProcess, baseAddress, ReturnBuffer, size, UIntPtr.Zero) ? ReturnBuffer : null;
        }
        public uint? SearchDAProcessMemory(uint searchItem) {
            byte[] srchlpBuffer;
            SafeNativeMethods.MEMORY_BASIC_INFORMATION mbi;

            uint lpMem = 0x00010000; //**32BIT ONLY** IntPtr lpMem = sys_info.lpMinimumApplicationAddress;
            UIntPtr lLenMPI = (UIntPtr)Marshal.SizeOf(typeof(SafeNativeMethods.MEMORY_BASIC_INFORMATION));

            while(lpMem < 0x7ffeffff) { //**32BIT ONLY** while((ulong)lpMem < (ulong)sys_info.lpMaximumApplicationAddress) {
                SafeNativeMethods.VirtualQueryEx(m_hProcess, (IntPtr)lpMem, out mbi, lLenMPI);
                //**FASTER** if (VirtualQueryEx(m_hProcess, (IntPtr)lpMem, out mbi, (UIntPtr)lLenMPI) == lLenMPI) {
                if(mbi.type == SafeNativeMethods.PageAccessFlags.MEM_PRIVATE && mbi.state == SafeNativeMethods.PageAccessFlags.MEM_COMMIT && mbi.protect == SafeNativeMethods.PageAccessFlags.PAGE_READWRITE) {
                    //**FASTER** if (mbi.RegionSize > 0) {
                    if((srchlpBuffer = ReadProcessMemory(mbi.baseAddress, mbi.regionSize)) == null) return null;
                    for(uint i = 0; i < (uint)mbi.regionSize; i = i + 4) if((srchlpBuffer[i] + 256 * srchlpBuffer[i + 1] + 256 * 256 * srchlpBuffer[i + 2]) == searchItem) return (uint)mbi.baseAddress + i; //**FASTER** + 256 * 256 * 256 * srchlpBuffer[i + 3]
                    //**FASTER** }
                }
                //**FASTER** }
                lpMem = (uint)mbi.baseAddress + (uint)mbi.regionSize;
            }
            return null;
        }

        private bool disposed;
        public void Dispose() { Dispose(true); GC.SuppressFinalize(this); }
        protected virtual void Dispose(bool disposing) {
            if(!disposed) {
                if(disposing && m_hProcess != null) m_hProcess.Dispose();
                disposed = true;
            }
        }
    }
}